@@ -1,11 +1,19 @@
 package HTML::TableParser::Table;
 
+# ABSTRACT: support class for HTML::TableParser
+
 use strict;
 use warnings;
 
-use HTML::Entities;
+use HTML::Entities qw();
 
-our $VERSION = '0.36';
+our $VERSION = '0.43';
+
+## no critic ( ProhibitAccessOfPrivateData )
+
+#pod =method new
+#pod
+#pod =cut
 
 sub new
 {
@@ -14,19 +22,19 @@ sub new
   my $class = ref($this) || $this;
 
   my $self = {
-	      data 	=> [[]],	# row data (for overlapping rows)
-	      row	=> undef,	# row info
-	      col	=> undef,	# column info
-	      hdr	=> undef,	# accumulated header info
-	      hdr_row 	=> 0,		# index of header row
-	      hdr_line	=> undef,	# line in file of header row
-	      in_hdr	=> 0,		# are we in a header row?
-	      prev_hdr	=> 0,		# was the previous row a header row?
-	      line	=> undef,	# line in file of current row
-	      start_line => undef,	# line in file of table start
-	      req	=> undef,	# the matching table request
-	      exclreqs  => {},		# the requests which exlude this table
-	     };
+              data      => [[]],        # row data (for overlapping rows)
+              row       => undef,       # row info
+              col       => undef,       # column info
+              hdr       => undef,       # accumulated header info
+              hdr_row   => 0,           # index of header row
+              hdr_line  => undef,       # line in file of header row
+              in_hdr    => 0,           # are we in a header row?
+              prev_hdr  => 0,           # was the previous row a header row?
+              line      => undef,       # line in file of current row
+              start_line => undef,      # line in file of table start
+              req       => undef,       # the matching table request
+              exclreqs  => {},          # the requests which exlude this table
+             };
 
   bless $self, $class;
 
@@ -59,13 +67,16 @@ sub new
     # for column name matches, we don't want to do the callback;
     # in that case $self->{req} isn't set and callback() won't
     # actually make the call.
-    $self->callback( 'start', $self->{start_line} ) 
-		     if $self->{process};
+    $self->callback( 'start', $self->{start_line} )
+                     if $self->{process};
   }
 
   $self;
 }
 
+#pod =method match_id
+#pod
+#pod =cut
 
 sub match_id
 {
@@ -84,8 +95,8 @@ sub match_id
   my ( $skip, $req );
 
   ( $skip, $req ) =
-    req_match_id( $self->{reqs}, $self->{id}, $self->{oids}, 
-		  $self->{exclreqs} );
+    req_match_id( $self->{reqs}, $self->{id}, $self->{oids},
+                  $self->{exclreqs} );
 
   # did we match a skip table request?
   return if $skip;
@@ -124,6 +135,10 @@ sub match_id
   # 4. out of luck. no match.
 }
 
+#pod =method req_match_id
+#pod
+#pod =cut
+
 # determine if a request matches an id.  requests should
 # be real objects, but until then...
 sub req_match_id
@@ -146,19 +161,19 @@ sub req_match_id
       # is this a subroutine to call?
       if ( 'CODE' eq ref $cmp->{match} )
       {
-	next unless $cmp->{match}->($id, $oids );
+        next unless $cmp->{match}->($id, $oids );
       }
 
       # regular expression
       elsif( 'Regexp' eq ref $cmp->{match} )
       {
-	next unless $id =~ /$cmp->{match}/;
+        next unless $id =~ /$cmp->{match}/;
       }
 
       # a direct match?
       else
       {
-	next unless $id eq $cmp->{match};
+        next unless $id eq $cmp->{match};
       }
 
       # we get here only if there was a match.
@@ -167,8 +182,8 @@ sub req_match_id
       # request.
       if ( $cmp->{exclude} )
       {
-	$excluded->{$req}++;
-	next;
+        $excluded->{$req}++;
+        next;
       }
 
       # return match, plus whether this is a global skip request
@@ -179,6 +194,10 @@ sub req_match_id
   ( 0, undef );
 }
 
+#pod =method req_match_cols
+#pod
+#pod =cut
+
 # determine if a request matches a column.  requests should
 # be real objects, but until then...
 sub req_match_cols
@@ -199,19 +218,19 @@ sub req_match_cols
       # is this a subroutine to call?
       if ( 'CODE' eq ref $cmp->{match} )
       {
-	next unless $cmp->{match}->( $id, $oids, \@fix_cols );
+        next unless $cmp->{match}->( $id, $oids, \@fix_cols );
       }
 
       # regular expression
       elsif( 'Regexp' eq ref $cmp->{match} )
       {
-	next unless grep { /$cmp->{match}/ } @fix_cols;
+        next unless grep { /$cmp->{match}/ } @fix_cols;
       }
 
       # a direct match?
       else
       {
-	next unless grep { $_ eq $cmp->{match} } @fix_cols;
+        next unless grep { $_ eq $cmp->{match} } @fix_cols;
       }
 
       # we get here only if there was a match
@@ -229,6 +248,10 @@ sub req_match_cols
   (0, undef);
 }
 
+#pod =method match_hdr
+#pod
+#pod =cut
+
 # we've pulled in a header; does it match against one of the requests?
 sub match_hdr
 {
@@ -241,7 +264,7 @@ sub match_hdr
 
   # 1. check header matches
   my ( $skip, $req ) = req_match_cols( $self->{reqs}, \@cols, $self->{id},
-				       $self->{oids} );
+                                       $self->{oids} );
   # did we match a skip table request?
   return 0 if $skip;
 
@@ -253,7 +276,7 @@ sub match_hdr
 
 
   # 2. DEFAULT match
-  ( $skip, $req ) = 
+  ( $skip, $req ) =
     req_match_id( $self->{reqs}, 'DEFAULT', $self->{oids}, $self->{exclreqs} );
 
   # did we match a skip table request? Does this make sense for DEFAULT?
@@ -270,6 +293,10 @@ sub match_hdr
   0;
 }
 
+#pod =method match_req
+#pod
+#pod =cut
+
 sub match_req
 {
   my ( $self, $req ) = @_;
@@ -291,6 +318,10 @@ sub match_req
 }
 
 
+#pod =method callback
+#pod
+#pod =cut
+
 # generic call back interface.  handle method calls as well as
 # subroutine calls.
 sub callback
@@ -298,7 +329,7 @@ sub callback
   my $self = shift;
   my $method = shift;
 
-  return unless 
+  return unless
     defined $self->{req} && exists $self->{req}->{$method};
 
   my $req = $self->{req};
@@ -320,6 +351,11 @@ sub callback
 
 
 # handle <th>
+
+#pod =method start_header
+#pod
+#pod =cut
+
 sub start_header
 {
   my $self = shift;
@@ -333,6 +369,11 @@ sub start_header
 
 
 # handle </th>
+
+#pod =method end_header
+#pod
+#pod =cut
+
 sub end_header
 {
   my $self = shift;
@@ -340,6 +381,10 @@ sub end_header
 }
 
 # handle <td>
+#pod =method start_column
+#pod
+#pod =cut
+
 sub start_column
 {
   my $self = shift;
@@ -352,8 +397,8 @@ sub start_column
   # we really shouldn't be here if a row hasn't been started
   unless ( defined $self->{row} )
   {
-    $self->callback( 'warn', $self->{id}, $line, 
-		     "<td> or <th> without <tr> at line $line\n" );
+    $self->callback( 'warn', $self->{id}, $line,
+                     "<td> or <th> without <tr> at line $line\n" );
     $self->start_row( {}, $line );
   }
 
@@ -374,6 +419,11 @@ sub start_column
 }
 
 # handle </td>
+
+#pod =method end_column
+#pod
+#pod =cut
+
 sub end_column
 {
   my $self = shift;
@@ -388,6 +438,10 @@ sub end_column
   $self->{text} = undef;
 }
 
+#pod =method start_row
+#pod
+#pod =cut
+
 sub start_row
 {
   my $self = shift;
@@ -401,6 +455,10 @@ sub start_row
 }
 
 
+#pod =method end_row
+#pod
+#pod =cut
+
 sub end_row
 {
   my $self = shift;
@@ -416,7 +474,6 @@ sub end_row
   {
 
     my $cn = 0;
-    my $rn = 0;
     foreach my $col ( @{$self->{row}} )
     {
       # do this just in case there are newlines and we're concatenating
@@ -431,14 +488,14 @@ sub end_row
       # note that header is stored as one array per column, not row!
       for ( my $cnn = 0 ; $cnn < $col->{attr}{colspan} ; $cnn++, $cn++ )
       {
-	$self->{hdr}[$cn] ||= [];
-	$self->{hdr}[$cn][$self->{hdr_row}] = $col->{text};
-	
-	# put empty placeholders in the rest of the rows
-	for ( my $rnn = 1 ; $rnn < $col->{attr}{rowspan} ; $rnn++ )
-	{
-	  $self->{hdr}[$cn][$rnn + $self->{hdr_row}] = '';
-	}
+        $self->{hdr}[$cn] ||= [];
+        $self->{hdr}[$cn][$self->{hdr_row}] = $col->{text};
+
+        # put empty placeholders in the rest of the rows
+        for ( my $rnn = 1 ; $rnn < $col->{attr}{rowspan} ; $rnn++ )
+        {
+          $self->{hdr}[$cn][$rnn + $self->{hdr_row}] = '';
+        }
       }
     }
 
@@ -447,7 +504,6 @@ sub end_row
   else
   {
     my $cn = 0;
-    my $rn = 0;
     foreach my $col ( @{$self->{row}} )
     {
       # need to find the first undefined column
@@ -455,11 +511,11 @@ sub end_row
 
       for ( my $cnn = 0 ; $cnn < $col->{attr}{colspan} ; $cnn++, $cn++ )
       {
-	for ( my $rnn = 0 ; $rnn < $col->{attr}{rowspan} ; $rnn++ )
-	{
-	  $self->{data}[$rnn] ||= [];
-	  $self->{data}[$rnn][$cn] = $col->{text};
-	}
+        for ( my $rnn = 0 ; $rnn < $col->{attr}{rowspan} ; $rnn++ )
+        {
+          $self->{data}[$rnn] ||= [];
+          $self->{data}[$rnn][$cn] = $col->{text};
+        }
       }
     }
   }
@@ -469,8 +525,8 @@ sub end_row
     if ! $self->{in_hdr} && $self->{prev_hdr};
 
   # output the data if we're not in a header
-  $self->callback( 'row', $self->{line}, 
-		   fix_texts( $self->{req}, shift @{$self->{data}} ) )
+  $self->callback( 'row', $self->{line},
+                   fix_texts( $self->{req}, shift @{$self->{data}} ) )
       unless $self->{in_hdr};
 
   $self->{in_hdr} = 0;
@@ -479,6 +535,11 @@ sub end_row
 
 # collect the possible multiple header rows into one array and
 # send it off
+
+#pod =method finish_header
+#pod
+#pod =cut
+
 sub finish_header
 {
   my $self = shift;
@@ -537,6 +598,10 @@ DESTROY
   }
 }
 
+#pod =method fix_texts
+#pod
+#pod =cut
+
 sub fix_texts
 {
   my ( $req, $texts  ) = @_;
@@ -549,10 +614,10 @@ sub fix_texts
     $HTML::Entities::entity2char{nbsp} = ' '
       if $req->{DecodeNBSP};
 
-    chomp $_ 
+    chomp $_
       if $req->{Chomp};
 
-    decode_entities( $_ )
+    HTML::Entities::decode_entities( $_ )
       if $req->{Decode};
 
 
@@ -566,6 +631,10 @@ sub fix_texts
   $texts;
 }
 
+#pod =method text
+#pod
+#pod =cut
+
 sub text
 {
   my $self = shift;
@@ -573,18 +642,47 @@ sub text
   $self->{text} = shift;
 }
 
+#pod =method id
+#pod
+#pod =cut
+
 sub id  { $_[0]->{id} }
+
+#pod =method ids
+#pod
+#pod =cut
+
 sub ids { $_[0]->{ids} }
+
+
+#pod =method process
+#pod
+#pod =cut
+
 sub process { $_[0]->{process} }
 
 1;
 
-__END__
+#
+# This file is part of HTML-TableParser
+#
+# This software is Copyright (c) 2018 by Smithsonian Astrophysical Observatory.
+#
+# This is free software, licensed under:
+#
+#   The GNU General Public License, Version 3, June 2007
+#
+
+=pod
 
 =head1 NAME
 
 HTML::TableParser::Table - support class for HTML::TableParser
 
+=head1 VERSION
+
+version 0.43
+
 =head1 DESCRIPTION
 
 This class is used to keep track of information related to a table and
@@ -614,19 +712,115 @@ The class handles missing C</tr>, C</td>, and C</th> tags.  As such
 be slightly delayed (and data cached).  It also handles rows
 with overlapping columns
 
-=head1 LICENSE
+=head1 METHODS
 
-This software is released under the GNU General Public License.  You
-may find a copy at 
+=head2 new
 
-   http://www.fsf.org/copyleft/gpl.html
+=head2 match_id
 
-=head1 AUTHOR
+=head2 req_match_id
 
-Diab Jerius (djerius@cpan.org)
+=head2 req_match_cols
+
+=head2 match_hdr
+
+=head2 match_req
+
+=head2 callback
+
+=head2 start_header
+
+=head2 end_header
+
+=head2 start_column
+
+=head2 end_column
+
+=head2 start_row
+
+=head2 end_row
+
+=head2 finish_header
+
+=head2 fix_texts
+
+=head2 text
+
+=head2 id
+
+=head2 ids
+
+=head2 process
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=HTML-TableParser> or by
+email to
+L<bug-HTML-TableParser@rt.cpan.org|mailto:bug-HTML-TableParser@rt.cpan.org>.
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
+=head1 SOURCE
+
+The development version is on github at L<https://github.com/djerius/html-tableparser>
+and may be cloned from L<git://github.com/djerius/html-tableparser.git>
 
 =head1 SEE ALSO
 
-L<HTML::Parser>, L<HTML::TableExtract>.
+Please see those modules/websites for more information related to this module.
+
+=over 4
+
+=item *
+
+L<HTML::TableParser|HTML::TableParser>
+
+=back
+
+=head1 AUTHOR
+
+Diab Jerius <djerius@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2018 by Smithsonian Astrophysical Observatory.
+
+This is free software, licensed under:
+
+  The GNU General Public License, Version 3, June 2007
 
 =cut
+
+__END__
+
+#pod =head1 DESCRIPTION
+#pod
+#pod This class is used to keep track of information related to a table and
+#pod to create the information passed back to the user callbacks.  It is in
+#pod charge of marshalling the massaged header and row data to the user
+#pod callbacks.
+#pod
+#pod An instance is created when the controlling TableParser class finds a
+#pod C<<table> tag.  The object is given an id based upon which table it is
+#pod to work on.  Its methods are invoked from the TableParser callbacks
+#pod when they run across an appropriate tag (C<tr>, C<th>, C<td>).  The
+#pod object is destroyed when the matching C</table> tag is found.
+#pod
+#pod Since tables may be nested, multiple B<HTML::TableParser::Table>
+#pod objects may exist simultaneously.  B<HTML::TableParser> uses two
+#pod pieces of information held by this class -- ids and process.  The
+#pod first is an array of table ids, one element per level of table
+#pod nesting.  The second is a flag indicating whether this table is being
+#pod processed (i.e. it matches a requested table) or being ignored.  Since
+#pod B<HTML::TableParser> uses the ids information from an existing table
+#pod to initialize a new table, it first creates an empty sentinel (place
+#pod holder) table (by calling the B<HTML::TableParser::Table> constructor
+#pod with no arguments).
+#pod
+#pod The class handles missing C</tr>, C</td>, and C</th> tags.  As such
+#pod (especially when handling multi-row headers) user callbacks may
+#pod be slightly delayed (and data cached).  It also handles rows
+#pod with overlapping columns
